
#include <stdio.h>
#include <math.h>
#include "mex.h"
#include "quicklz.h"

/*
  Compile to mex file with: 
  mex compress_mex_quicklz.c quicklz.c

  Nanne van der Zijpp
  Modelit
  www.modelit.nl
*/

#define NINPUT 1
#define NOUTPUT 1
//#define VERBOSE 1

static mxArray * do_compress(const mxArray *);

void mexFunction(int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[]) {
	
    if (nlhs != NOUTPUT) {
        mexErrMsgTxt("wrong number of output arguments");
    }
    if (nrhs != NINPUT) {
        mexErrMsgTxt("wrong number of input arguments");
    }
    
    plhs[0] = do_compress(prhs[0]);
}

/*
int compress (Bytef *dest, uLongf *destLen, const Bytef *source, uLong sourceLen); 
Compresses the source buffer into the destination buffer. 
sourceLen is the byte length of the source buffer. 
Upon entry, destLen is the total size of the destination buffer, which must be at least 0.1% larger than sourceLen plus 12 bytes. Upon exit, destLen is the actual size of the compressed buffer.
This function can be used to compress a whole file at once if the input file is mmap'ed.

  compress returns Z_OK if success, Z_MEM_ERROR if there was not enough memory, Z_BUF_ERROR if there was not enough room in the output buffer.
*/

static mxArray * do_compress(const mxArray * input)
{	
    size_t destLen[1],sourceLen, len;
    char *scratch, *p_output;	 
	mxArray * output;
	
	sourceLen = mxGetM(input)*mxGetN(input)*mxGetElementSize(input);
	
	*destLen = (size_t ) (sourceLen + 400);	
	
	output=mxCreateNumericMatrix(*destLen,1,mxINT8_CLASS,mxREAL);
	p_output=mxGetData(output);
    
    // QLZ_SCRATCH_COMPRESS is defined in the beginning of the quicklz.h file
    scratch = (char*) malloc(QLZ_SCRATCH_COMPRESS);
	
    // compress and write result
    len = qlz_compress((char *) mxGetData(input), p_output, sourceLen, scratch);
    
#ifdef VERBOSE
mexPrintf("Compressed from %d bytes to %d bytes\n", sourceLen, *destLen);
#endif
    mxSetM(output, len);
    return(output);
}

