function data_out=uncompressQuicklz(data_in,szOrg,class_name)
% uncompressQuicklz - low level function to uncompress byte vector
% 
% CALL
%     data_out=uncompressQuicklz(data_in,szOrg,class_name)
%     
% INPUT
%     data_in(P,1): int8 vector that contains compressed variable 
%     szOrg:        size of variable after decompressing.
%     class_name:   Required class of output array (defaults to double)
%                   Any of the following values are allowed:
%                       double -- Double precision floating point numeric array
%                                 (this is the traditional MATLAB matrix or array)
%                       logical-- Logical array
%                       char   -- Character array
%                       single -- Single precision floating-point numeric
%                                 array
%                       int8   -- 8-bit signed integer array
%                       uint8  -- 8-bit unsigned integer array
%                       int16  -- 16-bit signed integer array
%                       uint16 -- 16-bit unsigned integer array
%                       int32  -- 32-bit signed integer array
%                       uint32 -- 32-bit unsigned integer array
%                       int64  -- 64-bit signed integer array
%                       uint64 -- 64-bit unsigned integer array
%
% OUTPUT
%     data_out: uncompressed array
% 
% SEE ALSO
%     compressQuicklz

% Copyright (C) 2010
% Nanne van der Zijpp
% Modelit
% www.modelit.nl

if nargin==0
    data_out=[];
    return
end
if nargin<2
    szOrg=[10^6,1];
end
if nargin<3
    class_name='double'; %double array
end

% NOTE:
%     uncompress_mex_quicklz has the following input
%     
%     data_in
%         byte array with compressed data
%     N
%         number of elements to extract
%     ExampleValue
%         empty array that will be used to infer data type

if class_name(1)=='l' %Logical
    %uncompress uint8, then convert to logicals.
    %how many elements need to be uncompressed?
    %     prod(double(szOrg)) logicals ==>
    N=prod(double(szOrg));
    numbytes=ceil(N/8);
    %NOTE ZIJPP for logical: there was not enough room in the output
    %buffer. Compresses the source buffer into the destination buffer.
    %sourceLen is the byte length of the source buffer. Upon entry, destLen
    %is the total size of the destination buffer, which must be at least
    %0.1% larger than sourceLen plus 12 bytes. Upon exit, destLen is the
    %actual size of the compressed buffer.  <== look into this

    uint8Var=uncompress_mex_quicklz(data_in,numbytes,zeros([0,0],'uint8'));
    %convert to logical
    L=int2logical(uint8Var);
    data_out=reshape(...
        L(1:N),...
        szOrg);
    return
end
% data_out=zipUtil.uncompress(data_in,prod(szOrg),zeros([0,0]));
data_out=reshape(...
    uncompress_mex_quicklz(data_in,prod(double(szOrg)),zeros([0,0],class_name)),...
    szOrg);




