
#include <stdio.h>
#include <math.h>
#include <memory.h>
#include "mex.h"
#include "quicklz.h"


/*
  Compile to mex file with: 
  mex uncompress_mex_quicklz.c quicklz.c
  Nanne van der Zijpp
  Modelit
  www.modelit.nl
*/

#define NINPUT  3
#define NOUTPUT 1
//#define VERBOSE 1


static mxArray * do_uncompress(const mxArray *,const mxArray *, const mxArray *);

void mexFunction(int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[]) {
	
    if (nlhs != NOUTPUT) {
        mexErrMsgTxt("wrong number of output arguments");
    }
    if (nrhs != NINPUT) {
        mexErrMsgTxt("wrong number of input arguments");
    }
    
    plhs[0] = do_uncompress(prhs[0],prhs[1],prhs[2]);
}

/*

int uncompress (Bytef *dest, uLongf *destLen, const Bytef *source, uLong sourceLen); 
Decompresses the source buffer into the destination buffer. 
sourceLen is the byte length of the source buffer. 
Upon entry, destLen is the total size of the destination buffer, which must be large enough to hold the entire uncompressed data. (The size of the uncompressed data must have been saved previously by the compressor and transmitted to the decompressor by some mechanism outside the scope of this compression library.) Upon exit, destLen is the actual size of the compressed buffer. 
This function can be used to decompress a whole file at once if the input file is mmap'ed. 

uncompress returns: 
    Z_OK if success, 
	Z_MEM_ERROR if there was not enough memory, 
	Z_BUF_ERROR if there was not enough room in the output buffer, or 
	Z_DATA_ERROR if the input data was corrupted. 
*/

static mxArray * do_uncompress(const mxArray * input,const mxArray * N, const mxArray * example)
//input: compressed byte array
//N: number of elements to produce after inflating
//example: example Matlab array to infer element size (in bytes) and class of new array
{

	char *dest, *scratch;	
    unsigned int  numEl, sizeEl;
    unsigned int sourceLen;
    size_t destLen;
	int RC;
	
	mxArray * output;
	void *p_output;
	
	sourceLen = mxGetM(input)*mxGetN(input);
	/*
	Use empty array "example" to infer class of new variable
		    *destLen is the size of the output array in bytes.
			Now compute the number of elements
	*/
	sizeEl=mxGetElementSize(example); //element size in bytes
	destLen = (size_t) mxGetScalar(N)*sizeEl;
	numEl =destLen/sizeEl;           //number of elements
		
	dest=(char *) mxCalloc(destLen, sizeof(char)); // claim memory
	
    // QLZ_SCRATCH_DECOMPRESS is defined in the beginning of the quicklz.h file
    scratch = (char*) malloc(QLZ_SCRATCH_DECOMPRESS);
    
    destLen = qlz_decompress((char *) mxGetData(input), dest, scratch);
	
		//OK
#ifdef VERBOSE
		mexPrintf("Uncompressed from %d bytes to %d bytes\n",sourceLen,*destLen);
		mexPrintf("Create output matrix with %d elements of type %s\n",numEl,mxGetClassName(example));
#endif
		/*
		Possible classes to create:
			mxSINGLE_CLASS
			mxINT8_CLASS
			mxUINT8_CLASS
			mxINT16_CLASS
			mxUINT16_CLASS
			mxINT32_CLASS
			mxUINT32_CLASS
			mxINT64_CLASS
			mxUINT64_CLASS		
		*/

		output=mxCreateNumericMatrix(numEl,1,mxGetClassID(example),mxREAL);
		p_output=mxGetData(output); //void pointer to first element
		
		memcpy( p_output, dest, (size_t) destLen ); //copy contents of array

		mxFree(dest);
		return(output);
}



